@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -72,9 +72,9 @@ class ArgumentParser(argparse.ArgumentParser):
         name: The command name.
     """
 
-    def __init__(self, name, *args, **kwargs):
+    def __init__(self, name, **kwargs):
         self.name = name
-        super().__init__(*args, add_help=False, prog=name, **kwargs)
+        super().__init__(add_help=False, prog=name, **kwargs)
 
     def exit(self, status=0, message=None):
         raise ArgumentParserExit(status, message)
