@@ -23,7 +23,7 @@ import copy
 import contextlib
 import functools
 import typing
-from typing import Any, Optional
+from typing import Any
 
 from PyQt5.QtCore import pyqtSignal, QObject, QUrl
 
@@ -206,7 +206,7 @@ class KeyConfig:
                 'mode'.format(key, mode))
 
         self._validate(key, mode)
-        log.keyboard.vdebug(  # type: ignore
+        log.keyboard.vdebug(  # type: ignore[attr-defined]
             "Adding binding {} -> {} in mode {}.".format(key, command, mode))
 
         bindings = self._config.get_mutable_obj('bindings.commands')
