@@ -56,14 +56,14 @@ import typing
 import attr
 import yaml
 from PyQt5.QtCore import QUrl, Qt
-from PyQt5.QtGui import QColor, QFont, QFontDatabase
+from PyQt5.QtGui import QColor, QFontDatabase
 from PyQt5.QtWidgets import QTabWidget, QTabBar, QApplication
 from PyQt5.QtNetwork import QNetworkProxy
 
 from qutebrowser.misc import objects, debugcachestats
 from qutebrowser.config import configexc, configutils
 from qutebrowser.utils import (standarddir, utils, qtutils, urlutils, urlmatch,
-                               usertypes)
+                               usertypes, log)
 from qutebrowser.keyinput import keyutils
 from qutebrowser.browser.network import pac
 
@@ -1157,7 +1157,7 @@ class QssColor(BaseType):
 
 class FontBase(BaseType):
 
-    """Base class for Font/QtFont/FontFamily."""
+    """Base class for Font/FontFamily."""
 
     # Gets set when the config is initialized.
     default_family = None  # type: str
@@ -1292,96 +1292,6 @@ class FontFamily(FontBase):
         return value
 
 
-class QtFont(FontBase):
-
-    """A Font which gets converted to a QFont."""
-
-    __doc__ = Font.__doc__  # for src2asciidoc.py
-
-    def _parse_families(self, family_str: str) -> configutils.FontFamilies:
-        if family_str == 'default_family' and self.default_family is not None:
-            family_str = self.default_family
-
-        return configutils.FontFamilies.from_str(family_str)
-
-    def _set_style(self, font: QFont, match: typing.Match) -> None:
-        style = match.group('style')
-        style_map = {
-            'normal': QFont.StyleNormal,
-            'italic': QFont.StyleItalic,
-            'oblique': QFont.StyleOblique,
-        }
-        if style:
-            font.setStyle(style_map[style])
-        else:
-            font.setStyle(QFont.StyleNormal)
-
-    def _set_weight(self, font: QFont, match: typing.Match) -> None:
-        weight = match.group('weight')
-        namedweight = match.group('namedweight')
-        weight_map = {
-            'normal': QFont.Normal,
-            'bold': QFont.Bold,
-        }
-        if namedweight:
-            font.setWeight(weight_map[namedweight])
-        elif weight:
-            # based on qcssparser.cpp:setFontWeightFromValue
-            font.setWeight(min(int(weight) // 8, 99))
-        else:
-            font.setWeight(QFont.Normal)
-
-    def _set_size(self, font: QFont, match: typing.Match) -> None:
-        size = match.group('size')
-        if size:
-            if size == 'default_size':
-                size = self.default_size
-
-            if size is None:
-                # initial validation before default_size is set up.
-                pass
-            elif size.lower().endswith('pt'):
-                font.setPointSizeF(float(size[:-2]))
-            elif size.lower().endswith('px'):
-                font.setPixelSize(int(size[:-2]))
-            else:
-                # This should never happen as the regex only lets pt/px
-                # through.
-                raise ValueError("Unexpected size unit in {!r}!".format(
-                    size))  # pragma: no cover
-
-    def _set_families(self, font: QFont, match: typing.Match) -> None:
-        family_str = match.group('family')
-        families = self._parse_families(family_str)
-        if hasattr(font, 'setFamilies'):
-            # Added in Qt 5.13
-            font.setFamily(families.family)  # type: ignore[arg-type]
-            font.setFamilies(list(families))
-        else:  # pragma: no cover
-            font.setFamily(families.to_str(quote=False))
-
-    def to_py(self, value: _StrUnset) -> typing.Union[_UnsetNone, QFont]:
-        self._basic_py_validation(value, str)
-        if isinstance(value, usertypes.Unset):
-            return value
-        elif not value:
-            return None
-
-        match = self.font_regex.fullmatch(value)
-        if not match:  # pragma: no cover
-            # This should never happen, as the regex always matches everything
-            # as family.
-            raise configexc.ValidationError(value, "must be a valid font")
-
-        font = QFont()
-        self._set_style(font, match)
-        self._set_weight(font, match)
-        self._set_size(font, match)
-        self._set_families(font, match)
-
-        return font
-
-
 class Regex(BaseType):
 
     """A regular expression.
@@ -2012,6 +1922,16 @@ class NewTabPosition(String):
             ('last', "At the end."))
 
 
+class LogLevel(String):
+
+    """A logging level."""
+
+    def __init__(self, none_ok: bool = False) -> None:
+        super().__init__(none_ok=none_ok)
+        self.valid_values = ValidValues(*[level.lower()
+                                          for level in log.LOG_LEVELS])
+
+
 class Key(BaseType):
 
     """A name of a key."""
