@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2018 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2020 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -19,11 +19,16 @@
 
 """Keychain string displayed in the statusbar."""
 
+from PyQt5.QtCore import pyqtSlot
+
 from qutebrowser.mainwindow.statusbar import textbase
+from qutebrowser.utils import usertypes
 
 
 class KeyString(textbase.TextBase):
 
     """Keychain string displayed in the statusbar."""
 
-    pass
+    @pyqtSlot(usertypes.KeyMode, str)
+    def on_keystring_updated(self, _mode, keystr):
+        self.setText(keystr)
