@@ -29,7 +29,6 @@ try:
 except ImportError:
     hunter = None
 
-import os
 import sys
 import faulthandler
 import traceback
@@ -39,7 +38,7 @@ import datetime
 try:
     import tkinter
 except ImportError:
-    tkinter = None  # type: ignore
+    tkinter = None  # type: ignore[assignment]
 
 # NOTE: No qutebrowser or PyQt import should be done here, as some early
 # initialization needs to take place before that!
@@ -131,19 +130,6 @@ def init_faulthandler(fileobj=sys.__stderr__):
         # pylint: enable=no-member,useless-suppression
 
 
-def pyinstaller_qt_workaround():
-    """Work around PATH issues with PyInstaller.
-
-    See https://github.com/pyinstaller/pyinstaller/issues/4293
-    and https://github.com/gridsync/gridsync/pull/236/commits/0abf8e7363cc8c2a10a0263e6dcceb3be1c07022
-    """
-    if (hasattr(sys, 'frozen') and
-            hasattr(sys, '_MEIPASS') and
-            sys.platform == 'win32'):
-        # pylint: disable=no-member,protected-access
-        os.environ['PATH'] += os.pathsep + sys._MEIPASS  # type: ignore
-
-
 def check_pyqt():
     """Check if PyQt core modules (QtCore/QtWidgets) are installed."""
     for name in ['PyQt5.QtCore', 'PyQt5.QtWidgets']:
@@ -265,7 +251,8 @@ def configure_pyqt():
     from PyQt5 import QtCore
     QtCore.pyqtRemoveInputHook()
     try:
-        QtCore.pyqt5_enable_new_onexit_scheme(True)  # type: ignore
+        QtCore.pyqt5_enable_new_onexit_scheme(  # type: ignore[attr-defined]
+            True)
     except AttributeError:
         # Added in PyQt 5.13 somewhere, going to be the default in 5.14
         pass
@@ -273,7 +260,7 @@ def configure_pyqt():
     from qutebrowser.qt import sip
     try:
         # Added in sip 4.19.4
-        sip.enableoverflowchecking(True)  # type: ignore
+        sip.enableoverflowchecking(True)  # type: ignore[attr-defined]
     except AttributeError:
         pass
 
@@ -311,7 +298,6 @@ def early_init(args):
     init_faulthandler()
     # Here we check if QtCore is available, and if not, print a message to the
     # console or via Tk.
-    pyinstaller_qt_workaround()
     check_pyqt()
     # Init logging as early as possible
     init_log(args)
